#!/usr/bin/python

import netifaces as ni
import ipcalc
import sys
import os
import shutil
import argparse
import socket
import dns.resolver


parser = argparse.ArgumentParser()
parser.add_argument( "--reset",  default=False, help="Reset configuration files !",action="store_true")
args = parser.parse_args()
reset = args.reset

inf = ni.gateways()['default'][ni.AF_INET][1]
ip = ni.ifaddresses(inf)[ni.AF_INET][0]['addr']
mask = ni.ifaddresses(inf)[ni.AF_INET][0]['netmask']
prefix = sum([bin(int(x)).count('1') for x in mask.split('.')])
net = ipcalc.Network('%s/%s'%(ip, prefix)).network()
fqdn = socket.getfqdn()
domain = fqdn.partition('.')[2]

try:
    q = dns.resolver.query(fqdn, 'A')
except:
    print "You have to add %s in your DNS zone" % fqdn
    sys.exit(1)

if q[0].address != ip:
    print "You have to update %s with %s in your DNS zone" % (fqdn, ip)
    sys.exit(1)

proxy_pac = """
function FindProxyForURL(url, host) {
if (isInNet(host, "10.0.0.0", "255.0.0.0")) {	return "DIRECT";}
if (isInNet(host, "192.168.0.0", "255.255.0.0")) {	return "DIRECT";}
if (isInNet(host, "172.16.0.0", "255.240.0.0")) {	return "DIRECT";}
if (shExpMatch(host, "*.%s")) {return "DIRECT";}
if (isPlainHostName(host)) {return "DIRECT";}
if (isInNet(host, "127.0.0.0", "255.0.0.0")) { return "DIRECT";}
if (shExpMatch(host, "%s")) {return "DIRECT";}
     return "PROXY %s:8080";
}""" % ( domain, fqdn, fqdn )

proxy_pac_file = '/var/www/html/proxy.pac'

if not os.path.exists(proxy_pac_file) or reset:
    print "Reset Proxy.pac conf"
    with open(proxy_pac_file, "w") as f:
        f.write(proxy_pac)
ln = '/var/www/html/wpad.dat'
if not os.path.exists(ln) or reset:
    if os.path.exists(ln):
       os.unlink(ln)
    os.symlink(proxy_pac_file, ln)

if not os.path.exists("/opt/tis/tis-proxy-gui/tis-proxy-gui.ini") or reset:
    shutil.copy('/opt/tis/tis-proxy-gui/tis-proxy-gui.ini.distrib','/opt/tis/tis-proxy-gui/tis-proxy-gui.ini')

if not os.path.exists("opt/tis/tis-proxy-gui/exceptionsitelist") or reset:
    shutil.copy('/opt/tis/tis-proxy-gui/exceptionsitelist.tmpl', '/etc/e2guardian/lists/exceptionsitelist')

if not os.path.exists("/etc/e2guardian/lists/exceptionsitelist-local") or reset:
    open('/etc/e2guardian/lists/exceptionsitelist-local', 'w').close()

blacklists= '/var/lib/tis-proxy/blacklists/'
if not os.path.exists(blacklists+'webmail') or reset:
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.settimeout(10)
    result = sock.connect_ex(('ftp.univ-tlse1.fr',873))
    if not result == 0:
        print "You have to open rsync port (TCP 873) on your firewall"
        sys.exit(1)
    shutil.rmtree(blacklists)
    print "Get last Toulouse University blacklist DBs "
    os.system("python /opt/tis/tis-proxy-gui/scripts/update_blocklist_db.py")

e2guardian = """
addheaderregexplist = '/etc/e2guardian/lists/addheaderregexplist'
avadmin = ''
avsubject = 'e2guardian virus block'
bannedextensionlist = '/etc/e2guardian/lists/bannedextensionlist'
bannedmimetypelist = '/etc/e2guardian/lists/bannedmimetypelist'
bannedphraselist = '/etc/e2guardian/lists/bannedphraselist'
bannedregexpheaderlist = '/etc/e2guardian/lists/bannedregexpheaderlist'
bannedregexpurllist = '/etc/e2guardian/lists/bannedregexpurllist'
bannedsitelist = '/etc/e2guardian/lists/bannedsitelist'
bannedsslsitelist = '/etc/e2guardian/lists/bannedsslsitelist'
bannedurllist = '/etc/e2guardian/lists/bannedurllist'
blockdownloads = off
bypass = 0
bypasskey = ''
categorydisplaythreshold = 0
contentadmin = ''
contentregexplist = '/etc/e2guardian/lists/contentregexplist'
contentsubject = 'e2guardian violation'
deepurlanalysis = off
disablecontentscan = off
embeddedurlweight = 0
embededreferersitelist = '/etc/e2guardian/lists/embededreferersitelist'
embededrefererurllist = '/etc/e2guardian/lists/embededrefererurllist'
enablepics = off
exceptionextensionlist = '/etc/e2guardian/lists/exceptionextensionlist'
exceptionfilesitelist = '/etc/e2guardian/lists/exceptionfilesitelist'
exceptionfileurllist = '/etc/e2guardian/lists/exceptionfileurllist'
exceptionmimetypelist = '/etc/e2guardian/lists/exceptionmimetypelist'
exceptionphraselist = '/etc/e2guardian/lists/exceptionphraselist'
exceptionregexpurllist = '/etc/e2guardian/lists/exceptionregexpurllist'
exceptionsitelist = '/etc/e2guardian/lists/exceptionsitelist'
exceptionurllist = '/etc/e2guardian/lists/exceptionurllist'
greysitelist = '/etc/e2guardian/lists/greysitelist'
greysslsitelist = '/etc/e2guardian/lists/greysslsitelist'
greyurllist = '/etc/e2guardian/lists/greyurllist'
groupmode = 1
groupname = ''
headerregexplist = '/etc/e2guardian/lists/headerregexplist'
infectionbypass = 0
infectionbypasserrorsonly = on
infectionbypasskey = ''
mailfrom = ''
maxuploadsize = -1
mitmcheckcert = on
naughtynesslimit = 100000000
nocheckcertsitelist = '/etc/e2guardian/lists/nocheckcertsitelist'
notifyav = off
notifycontent = off
onlymitmsslgrey = off
picsfile = '/etc/e2guardian/lists/pics'
refererexceptionsitelist = '/etc/e2guardian/lists/refererexceptionsitelist'
refererexceptionurllist = '/etc/e2guardian/lists/refererexceptionurllist'
sslcertcheck = off
sslmitm = off
sslsiteregexplist = '/etc/e2guardian/lists/sslsiteregexplist'
threshold = 0
thresholdbyuser = off
urlredirectregexplist = '/etc/e2guardian/lists/urlredirectregexplist'
urlregexplist = '/etc/e2guardian/lists/urlregexplist'
usesmtp = off
violations = 0
weightedphraselist = '/etc/e2guardian/lists/weightedphraselist'
ssldeniedrewrite = off
reportinglevel = 3
sslaccessdeniedaddress = 'http://%s/block'
bannedsitelistwithbypass = '/etc/e2guardian/lists/bannedsitelistwithbypass'
""" % fqdn
e2guardian_file = "/etc/e2guardian/e2guardianf1.conf"
if os.path.exists(e2guardian_file) :
    os.rename(e2guardian_file,e2guardian_file + '.old')

print "Reset e2guardianf1.conf"
with open(e2guardian_file, "w") as f:
    f.write(e2guardian)

template = """
<html>
<head>
<link rel="icon" href="http://%s/static/images/favicon.ico">
<title>Proxy</title>
<meta http-equiv="content-type" content="text/html; charset=utf-8" />
<meta http-equiv="refresh" content="1; URL=http://%s/block/?DENIEDURL==-URL-::IP==-IP-::USER==-USER-::CATEGORIES==-CATEGORIES-::REASON==-REASONGIVEN-">
</head>
<body>
</body>
</html>
""" % (fqdn, fqdn)
template_file = "/usr/share/e2guardian/languages/french/template.html"
if not os.path.exists(template_file) or reset:
    print "Reset Dansguardain template"
    with open(template_file, "w") as f:
        f.write(template)
    os.system('service e2guardian restart')

