#!/usr/bin/python
# -*- coding: utf8 -*-
import ConfigParser
import os
import sys
root_dir = os.path.abspath(os.path.join(os.path.dirname(__file__)))
sys.path.insert(0, os.path.join(root_dir))
sys.path.insert(0, os.path.join(root_dir, 'lib'))

import flask_login

import socket
import subprocess
import time
import urllib as ul
from datetime import datetime
from urlparse import urlparse
from urllib import unquote_plus

from flask import request, Flask, redirect, render_template, jsonify, url_for
from passlib.hash import sha256_crypt

app = Flask(__name__)
login_manager = flask_login.LoginManager()
login_manager.init_app(app)
app.config["SECRET_KEY"] = "AwgymlBSf7$5WPrLHiWkP7TzNDJz5Ul8Dq9PIQ"
config = ConfigParser.ConfigParser()
config.read('tis-proxy-gui.ini')

USER = config.get('general', 'USER')
admin_password = config.get('general', 'admin_password')
unblock_password = config.get('general', 'unblock_password')

exceptionsitelist = '/etc/e2guardian/lists/exceptionsitelist-local'
exceptionsitelist_tis = '/etc/e2guardian/lists/exceptionsitelist-tis'
exceptioniplist = '/etc/e2guardian/lists/exceptioniplist'
tis_blacklist="https://dev.tranquil.it/blacklist-tis"
tis_blacklist_file="/etc/e2guardian/lists/blacklist-tis"

ok_file = '/tmp/update_blocklist_db_status_ok'
ko_file = '/tmp/update_blocklist_db_status_ko'

users = {USER: {'pw': admin_password}}

class User(flask_login.UserMixin):
    pass

@login_manager.user_loader
def user_loader(login):
    if login not in users:
        return

    user = User()
    user.id = login
    return user

@login_manager.unauthorized_handler
def unauthorized_handler():
    return redirect(url_for('login'))

@app.route('/login/', methods=['GET', 'POST'])
def login():
    if request.method == 'GET':
        return render_template('login.html')
    login = request.form['login']
    if not users.has_key(login):
        return render_template('login.html', errors="Mauvais identifiant")
    if  sha256_crypt.verify(request.form['pw'], users[login]['pw']):
        user = User()
        user.id = login
        flask_login.login_user(user)
        return redirect(url_for('admin'))
    else:
        return render_template('login.html', errors="Mauvais mot de passe")

@app.route('/logout/')
def logout():
    flask_login.logout_user()
    return redirect(url_for('admin'))

@app.route('/ping/')
def status():
    return jsonify({'status': 'ok'})


@app.route('/block/')
def block():
    val = request.url.split('?')[1]
    deniedurl = ''
    reason = ''
    errors = None
    for i in val.split('::'):
        key = i.split('==')[0]
        value = ul.unquote_plus(i.split('==')[1])
        if key == 'DENIEDURL':
            deniedurl = value
        if key == 'CATEGORIES':
            categories = value
        if key == 'IP':
            source_ip = value
        if key == 'REASON':
            reason = value
        if key == 'USER':
            source_user = value
        if key == 'ERROR':
            errors = "Mauvais mot de passe"
    return render_template('message.html', deniedurl=deniedurl, reason=reason,errors=errors)


@app.route('/unblock/',methods=['POST'])
def unblock():
    if request.form.has_key('deniedurl') and request.form.has_key('password') and sha256_crypt.verify(
            request.form['password'], unblock_password):
        unblock_url(request.form['deniedurl'])
        reload_config()
        return redirect(request.form['deniedurl'])
    else:
        return redirect(request.referrer+"::ERROR==ERROR")


@app.route('/admin/', methods=['GET', 'POST'])
@flask_login.login_required
def admin():
    msg = ''
    if request.method == 'POST':
        if request.form.has_key('urls'):
            urls = request.form['urls'].splitlines()
            for url in urls:
                if not urlparse(url).hostname:
                    url = 'http://' + url
                unblock_url(url)
                reload_config()
            msg = u"Site débloqué avec succès !"
        if request.form.has_key('ip'):
            ip = request.form['ip']
            ltime = request.form['time']
            try:
                socket.inet_aton(ip)
                unblock_ip(ip, ltime)
                msg = u"IP %s autorisée pour %sH" % (ip, ltime)
            except socket.error:
                pass

    with open(exceptionsitelist) as f:
        sitelist = f.readlines()
    with open(exceptionsitelist_tis) as f:
        sitelist += f.readlines()

    if os.path.exists(ok_file):
        rsync_status = "OK"
        with open(ok_file) as f:
            rsync_msg = f.readlines()
            ctime = datetime.fromtimestamp(os.stat(ok_file)[-1]).strftime('%d-%m-%Y  %H:%M')
    elif os.path.exists(ko_file):
        rsync_status = "KO"
        with open(ko_file) as f:
            rsync_msg = f.readlines()
            ctime = datetime.fromtimestamp(os.stat(ko_file)[-1]).strftime('%d-%m-%Y  %H:%M')
    else:
        rsync_status = "UNKNOWN"
        rsync_msg = ""
        ctime = ""


    return render_template('admin.html', sitelist=sitelist, ip=request.access_route[0], msg=msg,
                       rsync=(rsync_status, rsync_msg, ctime))


def reload_config():
    # e2guardian need to be restart
    subprocess.call("service e2guardian restart" ,shell=True)


def unblock_url(deniedurl):
    hostname = urlparse(deniedurl).hostname
    if hostname in open("/etc/e2guardian/lists/blacklist-tis").read():
        return "Not possible to override it"
    if hostname in open(exceptionsitelist).read() or hostname in open(exceptionsitelist_tis).read():
        return "Already exist"
    #print "Unblock: " + deniedurl
    with open(exceptionsitelist, "a") as f:
        f.write(hostname + '\n')


def unblock_ip(ip, hours):
    if ip in open(exceptioniplist).read():
        return "Already exist"
    print "Allow temporary: " + ip
    with open(exceptioniplist, "a") as f:
        f.write('#EXCLUDE IP ' + str(time.time() + 3600 * int(hours)) + '\n' + ip + '\n')
    reload_config()


if __name__ == '__main__':
    app.run(host="127.0.0.1", port=8081, debug=True)
